@@ -259,15 +259,15 @@ static void
 gegl_node_dispose (GObject *gobject)
 {
   GeglNode *self = GEGL_NODE (gobject);
-
+  self->inert = TRUE;
   if (self->priv->parent != NULL)
     {
       GeglNode *parent = self->priv->parent;
       self->priv->parent = NULL;
       gegl_node_remove_child (parent, self);
     }
-
   gegl_node_remove_children (self);
+  self->is_graph = FALSE;
   g_clear_object (&self->cache);
   g_clear_object (&self->priv->eval_manager);
 
@@ -469,6 +469,38 @@ gegl_node_list_output_pads (GeglNode *self)
   return _make_pad_list (self->output_pads);
 }
 
+const gchar *
+gegl_node_get_pad_label (GeglNode    *node,
+                         const gchar *pad_name)
+{
+  GeglPad    *pad;
+  GParamSpec *spec;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+  g_return_val_if_fail (gegl_node_has_pad (node, pad_name), NULL);
+
+  pad = gegl_node_get_pad (node, pad_name);
+  spec = gegl_pad_get_param_spec (pad);
+
+  return g_param_spec_get_nick (spec);
+}
+
+const gchar *
+gegl_node_get_pad_description (GeglNode    *node,
+                               const gchar *pad_name)
+{
+  GeglPad    *pad;
+  GParamSpec *spec;
+
+  g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
+  g_return_val_if_fail (gegl_node_has_pad (node, pad_name), NULL);
+
+  pad = gegl_node_get_pad (node, pad_name);
+  spec = gegl_pad_get_param_spec (pad);
+
+  return g_param_spec_get_blurb (spec);
+}
+
 /**
  * gegl_node_get_pads:
  * @self: a #GeglNode.
@@ -1815,7 +1847,7 @@ gegl_node_get_producer (GeglNode     *node,
   GeglNode *ret;
   gpointer pad;
 
-  if (!GEGL_IS_NODE (node))
+  if (!GEGL_IS_NODE (node) || node->inert)
     return NULL;
 
   /* XXX: there should be public API to test if a node is
