@@ -11,9 +11,10 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
- * Copyright 2007 Øyvind Kolås
+ * Copyright 2007-2012,2014,2015,2017 Øyvind Kolås
+ *           2015                     Debarshi Ray 
  */
 
 #include "config.h"
@@ -21,18 +22,21 @@
 #include <glib-object.h>
 
 #include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-debug.h"
-#include "buffer/gegl-region.h"
-#include "graph/gegl-node.h"
+#include "gegl-region.h"
+#include "graph/gegl-node-private.h"
 
+#include "operation/gegl-operation-context.h"
+#include "operation/gegl-operation-context-private.h"
 #include "operation/gegl-operation-sink.h"
 
 #include "gegl-config.h"
 #include "gegl-processor.h"
-#include "gegl-types-internal.h"
-#include "gegl-utils.h"
+#include "gegl-processor-private.h"
 
 #include "graph/gegl-visitor.h"
+#include "graph/gegl-callback-visitor.h"
 #include "graph/gegl-visitable.h"
 
 #include "opencl/gegl-cl.h"
@@ -60,9 +64,7 @@ static void      gegl_processor_get_property (GObject               *gobject,
                                               GParamSpec            *pspec);
 static void      gegl_processor_set_node     (GeglProcessor         *processor,
                                               GeglNode              *node);
-static GObject * gegl_processor_constructor  (GType                  type,
-                                              guint                  n_params,
-                                              GObjectConstructParam *params);
+static void      gegl_processor_constructed  (GObject               *object);
 static gdouble   gegl_processor_progress     (GeglProcessor         *processor);
 static gint      gegl_processor_get_band_size(gint                   size) G_GNUC_CONST;
 
@@ -71,8 +73,11 @@ struct _GeglProcessor
 {
   GObject          parent;
   GeglNode        *node;
+  GeglNode        *real_node;
   GeglRectangle    rectangle;
+  GeglRectangle    rectangle_unscaled;
   GeglNode        *input;
+  gint             level;
   GeglOperationContext *context;
 
   GeglRegion      *valid_region;     /* used when doing unbuffered rendering */
@@ -93,7 +98,7 @@ gegl_processor_class_init (GeglProcessorClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize     = gegl_processor_finalize;
-  gobject_class->constructor  = gegl_processor_constructor;
+  gobject_class->constructed  = gegl_processor_constructed;
   gobject_class->set_property = gegl_processor_set_property;
   gobject_class->get_property = gegl_processor_get_property;
 
@@ -103,90 +108,71 @@ gegl_processor_class_init (GeglProcessorClass *klass)
                                                         "The GeglNode to process (will saturate the provider's cache if the provided node is a sink node)",
                                                         GEGL_TYPE_NODE,
                                                         G_PARAM_WRITABLE |
+                                                        G_PARAM_STATIC_STRINGS |
                                                         G_PARAM_CONSTRUCT));
 
   g_object_class_install_property (gobject_class, PROP_RECTANGLE,
                                    g_param_spec_pointer ("rectangle",
                                                          "rectangle",
                                                          "The rectangle of the region to process.",
-                                                         G_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE |
+                                                         G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_PROGRESS,
                                    g_param_spec_double ("progress",
                                                         "progress",
                                                         "query progress; 0.0 is not started, 1.0 is done.",
                                                         0.0, 1.0, 0.0,
-                                                        G_PARAM_READWRITE));
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_CHUNK_SIZE,
                                    g_param_spec_int ("chunksize",
                                                      "chunksize",
                                                      "Size of chunks being rendered (larger chunks need more memory to do the processing).",
-                                                     1, 1024 * 1024, gegl_config()->chunk_size,
+                                                     1, 4096 * 4096, gegl_config()->chunk_size,
                                                      G_PARAM_READWRITE |
+                                                     G_PARAM_STATIC_STRINGS |
                                                      G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
 gegl_processor_init (GeglProcessor *processor)
 {
+  processor->level            = 0;
   processor->node             = NULL;
+  processor->real_node        = NULL;
   processor->input            = NULL;
   processor->context          = NULL;
   processor->queued_region    = NULL;
   processor->dirty_rectangles = NULL;
-  processor->chunk_size       = 128 * 128;
+  //processor->chunk_size       = 128 * 128;
 }
 
-/* Initialises the fields processor->input, processor->valid_region
- * and processor->queued_region.
- */
-static GObject *
-gegl_processor_constructor (GType                  type,
-                            guint                  n_params,
-                            GObjectConstructParam *params)
+static void
+gegl_processor_constructed (GObject *object)
 {
-  GObject       *object;
-  GeglProcessor *processor;
+  GeglProcessor *processor = GEGL_PROCESSOR (object);
 
-  object    = G_OBJECT_CLASS (gegl_processor_parent_class)->constructor (type, n_params, params);
-  processor = GEGL_PROCESSOR (object);
+  G_OBJECT_CLASS (gegl_processor_parent_class)->constructed (object);
 
   processor->queued_region = gegl_region_new ();
-
-  return object;
 }
 
+
 static void
 gegl_processor_finalize (GObject *self_object)
 {
   GeglProcessor *processor = GEGL_PROCESSOR (self_object);
 
-  if (processor->context)
-    {
-      GeglCache *cache = gegl_node_get_cache (processor->input);
-      gegl_node_remove_context (processor->node, cache);
-    }
+  g_clear_pointer (&processor->context, gegl_operation_context_destroy);
 
-  if (processor->node)
-    {
-      g_object_unref (processor->node);
-    }
+  g_clear_object (&processor->node);
+  g_clear_object (&processor->real_node);
+  g_clear_object (&processor->input);
 
-  if (processor->input)
-    {
-      g_object_unref (processor->input);
-    }
-
-  if (processor->queued_region)
-    {
-      gegl_region_destroy (processor->queued_region);
-    }
-
-  if (processor->valid_region)
-    {
-      gegl_region_destroy (processor->valid_region);
-    }
+  g_clear_pointer (&processor->queued_region, gegl_region_destroy);
+  g_clear_pointer (&processor->valid_region, gegl_region_destroy);
 
   G_OBJECT_CLASS (gegl_processor_parent_class)->finalize (self_object);
 }
@@ -255,19 +241,28 @@ gegl_processor_set_node (GeglProcessor *processor,
                          GeglNode      *node)
 {
   g_return_if_fail (GEGL_IS_NODE (node));
-  g_return_if_fail (GEGL_IS_OPERATION (node->operation));
+  g_return_if_fail (node->is_graph || GEGL_IS_OPERATION (node->operation));
 
-  if (processor->node)
-    g_object_unref (processor->node);
-  processor->node = g_object_ref (node);
+  g_set_object (&processor->node, node);
+  g_clear_object (&processor->real_node);
+
+  /* nodes with meta operations are also graphs and can be sinks, so
+   * we don't use their output proxy */
+  if (GEGL_IS_OPERATION (node->operation))
+    processor->real_node = node;
+  else /* ie. node->is_graph */
+    processor->real_node = gegl_node_get_output_proxy (node, "output");
+
+  g_return_if_fail (processor->real_node != NULL);
+  g_object_ref (processor->real_node);
 
   /* if the processor's node is a sink operation then get the producer node
    * and set up the region (unless all is going to be needed) */
-  if (processor->node->operation &&
-      g_type_is_a (G_OBJECT_TYPE (processor->node->operation),
+  if (processor->real_node->operation &&
+      g_type_is_a (G_OBJECT_TYPE (processor->real_node->operation),
                    GEGL_TYPE_OPERATION_SINK))
     {
-      processor->input = gegl_node_get_producer (processor->node, "input", NULL);
+      processor->input = gegl_node_get_producer (processor->real_node, "input", NULL);
 
       if (processor->input == NULL)
         {
@@ -276,7 +271,7 @@ gegl_processor_set_node (GeglProcessor *processor,
           return;
         }
 
-      if (!gegl_operation_sink_needs_full (processor->node->operation))
+      if (!gegl_operation_sink_needs_full (processor->real_node->operation))
         {
           processor->valid_region = gegl_region_new ();
         }
@@ -289,18 +284,28 @@ gegl_processor_set_node (GeglProcessor *processor,
    * an input, and set the region to NULL */
   else
     {
-      processor->input = processor->node;
+      processor->input = processor->real_node;
       processor->valid_region = NULL;
     }
 
   g_return_if_fail (processor->input != NULL);
 
+  /* Prepare the graph */
+  gegl_node_get_bounding_box (processor->input);
+
   g_object_ref (processor->input);
 
   g_object_notify (G_OBJECT (processor), "node");
 }
 
-
+static void
+set_scaled_rectangle (GeglProcessor *processor)
+{
+  processor->rectangle.x = processor->rectangle_unscaled.x >> processor->level;
+  processor->rectangle.y = processor->rectangle_unscaled.y >> processor->level;
+  processor->rectangle.width = processor->rectangle_unscaled.width >> processor->level;
+  processor->rectangle.height = processor->rectangle_unscaled.height >> processor->level;
+}
 
 
 /* Sets the processor->rectangle to the given rectangle (or the node
@@ -329,7 +334,7 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
 
   /* if the processor's rectangle isn't already set to the node's bounding box,
    * then set it and remove processor->dirty_rectangles (set to NULL)  */
-  if (! gegl_rectangle_equal (&processor->rectangle, rectangle))
+  if (! gegl_rectangle_equal (&processor->rectangle_unscaled, rectangle))
     {
 #if 0
     /* XXX: this is a large penalty hit, so we assume the rectangle
@@ -337,9 +342,10 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
       GeglRectangle  bounds;
       bounds               = processor->bounds;/*gegl_node_get_bounding_box (processor->input);*/
 #endif
-      processor->rectangle = *rectangle;
+      processor->rectangle_unscaled = *rectangle;
+      set_scaled_rectangle (processor);
 #if 0
-      gegl_rectangle_intersect (&processor->rectangle, &processor->rectangle, &bounds);
+      gegl_rectangle_intersect (&processor->rectangle_unscaled, &processor->rectangle_unscaled, &bounds);
 #endif
     }
     {
@@ -356,28 +362,25 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
   /* if the node's operation is a sink and it needs the full content then
    * a context will be set up together with a cache and
    * needed and result rectangles */
-  if (processor->node &&
-      GEGL_IS_OPERATION_SINK (processor->node->operation) &&
-      gegl_operation_sink_needs_full (processor->node->operation))
+  if (processor->real_node &&
+      GEGL_IS_OPERATION_SINK (processor->real_node->operation) &&
+      gegl_operation_sink_needs_full (processor->real_node->operation))
     {
       GeglCache *cache;
-      GValue     value = { 0, };
 
       cache = gegl_node_get_cache (processor->input);
 
-      if (!gegl_node_get_context (processor->node, cache))
-        processor->context = gegl_node_add_context (processor->node, cache);
-
-      g_value_init (&value, GEGL_TYPE_BUFFER);
-      g_value_set_object (&value, cache);
-      gegl_operation_context_set_property (processor->context, "input", &value);
-      g_value_unset (&value);
+      if (!processor->context)
+        {
+          processor->context = gegl_operation_context_new (processor->real_node->operation, NULL);
+        }
 
+      gegl_operation_context_set_object (processor->context, "input", G_OBJECT (cache));
 
       gegl_operation_context_set_result_rect (processor->context,
-                                              &processor->rectangle);
+                                              &processor->rectangle_unscaled);
       gegl_operation_context_set_need_rect   (processor->context,
-                                              &processor->rectangle);
+                                              &processor->rectangle_unscaled);
     }
 
   if (processor->valid_region)
@@ -399,7 +402,12 @@ gegl_processor_get_band_size (gint size)
 
   /* try to make the rects generated match better with potential 2^n sized
    * tiles, XXX: should be improved to make the next slice fit as well. */
-  if (band_size <= 256)
+  if (band_size <= 128)
+    {
+      band_size = MIN(band_size, 64); /* prefer a band_size of 128,
+                                          hoping to hit tiles */
+    }
+  else if (band_size <= 256)
     {
       band_size = MIN(band_size, 128); /* prefer a band_size of 128,
                                           hoping to hit tiles */
@@ -423,19 +431,19 @@ gegl_processor_get_band_size (gint size)
 static gboolean
 render_rectangle (GeglProcessor *processor)
 {
-  gboolean   buffered;
-  const gint max_area = processor->chunk_size;
-  GeglCache *cache    = NULL;
-  gint       pxsize;
+  gboolean    buffered;
+  const gint  max_area = processor->chunk_size * (1<<processor->level) * (1<<processor->level) * gegl_config_threads();
+  GeglCache  *cache    = NULL;
+  const Babl *format   = NULL;
 
-  /* Retreive the cache if the processor's node is not buffered if it's
+  /* Retrieve the cache if the processor's node is not buffered if its
    * operation is a sink and it doesn't use the full area  */
-  buffered = !(GEGL_IS_OPERATION_SINK(processor->node->operation) &&
-               !gegl_operation_sink_needs_full (processor->node->operation));
+  buffered = !(GEGL_IS_OPERATION_SINK(processor->real_node->operation) &&
+               !gegl_operation_sink_needs_full (processor->real_node->operation));
   if (buffered)
     {
       cache = gegl_node_get_cache (processor->input);
-      g_object_get (cache, "px-size", &pxsize, NULL);
+      format = gegl_buffer_get_format ((GeglBuffer *)cache);
     }
 
   if (processor->dirty_rectangles)
@@ -486,38 +494,35 @@ render_rectangle (GeglProcessor *processor)
 
       if (buffered)
         {
-          /* only do work if the rectangle is not completely inside the valid
-           * region of the cache */
-          if (gegl_region_rect_in (cache->valid_region, dr) !=
-              GEGL_OVERLAP_RECTANGLE_IN)
+          gboolean found_full = FALSE;
+          for (gint level = processor->level; level >= 0; level--)
+          {
+            if (gegl_region_rect_in (cache->valid_region[level], dr) == GEGL_OVERLAP_RECTANGLE_IN)
             {
-              /* create a buffer and initialise it */
-              guchar *buf;
-
-              gegl_region_union_with_rect (cache->valid_region, dr);
-              buf = g_malloc (dr->width * dr->height * pxsize);
-              g_assert (buf);
+              found_full = TRUE;
+              break;
+            }
+            /* XXX: dr should be adjusted to be the bounding box of not-found
+             * in cache if there is partial hits
+             */
+          }
 
+          if (!found_full)
+            {
               /* do the image calculations using the buffer */
-              gegl_node_blit (cache->node, 1.0, dr, cache->format, buf,
-                              GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
-
-
-              /* copy the buffer data into the cache */
-              gegl_buffer_set (GEGL_BUFFER (cache), dr, 0, cache->format, buf,
-                               GEGL_AUTO_ROWSTRIDE); /* XXX: deal with the level */
+              gegl_node_blit (processor->input, 1.0/(1<<processor->level),
+                              dr, format, NULL,
+                              GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_CACHE);
 
               /* tells the cache that the rectangle (dr) has been computed */
-              gegl_cache_computed (cache, dr);
-
-              /* release the buffer */
-              g_free (buf);
+              gegl_cache_computed (cache, dr, processor->level);
             }
           g_slice_free (GeglRectangle, dr);
         }
       else
         {
-           gegl_node_blit (processor->node, 1.0, dr, NULL, NULL,
+           gegl_node_blit (processor->real_node, 1.0/(1<<processor->level),
+                           dr, NULL, NULL,
                            GEGL_AUTO_ROWSTRIDE, GEGL_BLIT_DEFAULT);
            gegl_region_union_with_rect (processor->valid_region, dr);
            g_slice_free (GeglRectangle, dr);
@@ -595,7 +600,7 @@ gegl_processor_progress (GeglProcessor *processor)
     }
   else
     {
-      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+      valid_region = gegl_node_get_cache (processor->input)->valid_region[processor->level];
     }
 
   wanted = rect_area (&(processor->rectangle));
@@ -635,7 +640,7 @@ gegl_processor_render (GeglProcessor *processor,
   else
     {
       g_return_val_if_fail (processor->input != NULL, FALSE);
-      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+      valid_region = gegl_node_get_cache (processor->input)->valid_region[processor->level];
     }
 
   {
@@ -740,6 +745,14 @@ gegl_processor_render (GeglProcessor *processor,
   return !gegl_processor_is_rendered (processor);
 }
 
+static gboolean
+gegl_processor_work_is_opencl_node (GeglNode *node,
+                                    gpointer  data)
+{
+  return GEGL_OPERATION_GET_CLASS (node->operation)->cl_data ||
+         GEGL_OPERATION_GET_CLASS (node->operation)->opencl_support;
+}
+
 /* Will call gegl_processor_render and when there is no more work to be done,
  * it will write the result to the destination */
 gboolean
@@ -747,29 +760,22 @@ gegl_processor_work (GeglProcessor *processor,
                      gdouble       *progress)
 {
   gboolean   more_work = FALSE;
-  GeglCache *cache     = gegl_node_get_cache (processor->input);
 
-  if (gegl_cl_is_accelerated () && gegl_config()->use_opencl
-      && processor->chunk_size != GEGL_CL_CHUNK_SIZE)
+  if (gegl_config()->use_opencl)
     {
-      GeglVisitor *visitor = g_object_new (GEGL_TYPE_VISITOR, NULL);
-      GSList *iterator = NULL;
-      GSList *visits_list = NULL;
-      gegl_visitor_reset (visitor);
-      gegl_visitor_dfs_traverse (visitor, GEGL_VISITABLE (processor->node));
-      visits_list = gegl_visitor_get_visits_list (visitor);
-
-      for (iterator = visits_list; iterator; iterator = iterator->next)
+      if (gegl_cl_is_accelerated ()
+          && processor->chunk_size != GEGL_CL_CHUNK_SIZE)
         {
-          GeglNode *node = (GeglNode*) iterator->data;
-          if (GEGL_OPERATION_GET_CLASS(node->operation)->opencl_support)
-            {
-              processor->chunk_size = GEGL_CL_CHUNK_SIZE;
-              break;
-            }
-        }
+          GeglVisitor *visitor;
 
-      g_object_unref (visitor);
+          visitor = gegl_callback_visitor_new (gegl_processor_work_is_opencl_node,
+                                               NULL);
+
+          if (gegl_visitor_traverse (visitor, GEGL_VISITABLE (processor->real_node)))
+            processor->chunk_size = GEGL_CL_CHUNK_SIZE;
+
+          g_object_unref (visitor);
+        }
     }
 
   more_work = gegl_processor_render (processor, &processor->rectangle, progress);
@@ -786,11 +792,11 @@ gegl_processor_work (GeglProcessor *processor,
   if (processor->context)
     {
       /* the actual writing to the destination */
-      gegl_operation_process (processor->node->operation,
+      gegl_operation_process (processor->real_node->operation,
                               processor->context,
                               "output"  /* ignored output_pad */,
                               &processor->context->result_rect, processor->context->level);
-      gegl_node_remove_context (processor->node, cache);
+      gegl_operation_context_destroy (processor->context);
       processor->context = NULL;
 
       return TRUE;
@@ -810,3 +816,23 @@ gegl_node_new_processor (GeglNode            *node,
                        "rectangle", rectangle,
                        NULL);
 }
+
+void gegl_processor_set_level (GeglProcessor *processor,
+                               gint           level)
+{
+  processor->level = level;
+  set_scaled_rectangle (processor);
+}
+
+GeglBuffer *gegl_processor_get_buffer (GeglProcessor *processor)
+{
+  return processor?GEGL_BUFFER(gegl_node_get_cache (processor->input)):NULL;
+}
+
+
+void gegl_processor_set_scale (GeglProcessor *processor,
+                               gdouble        scale)
+{
+  processor->level = gegl_level_from_scale (scale);
+  set_scaled_rectangle (processor);
+}
